# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'CustomField'
        db.create_table('custom_field_customfield', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=75)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('field_type', self.gf('django.db.models.fields.CharField')(default='t', max_length=1)),
        ))
        db.send_create_signal('custom_field', ['CustomField'])

        # Adding unique constraint on 'CustomField', fields ['name', 'content_type']
        db.create_unique('custom_field_customfield', ['name', 'content_type_id'])

        # Adding model 'CustomFieldValue'
        db.create_table('custom_field_customfieldvalue', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('field', self.gf('django.db.models.fields.related.ForeignKey')(related_name='instance', to=orm['custom_field.CustomField'])),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'], null=True, blank=True)),
        ))
        db.send_create_signal('custom_field', ['CustomFieldValue'])

        # Adding unique constraint on 'CustomFieldValue', fields ['field', 'object_id']
        db.create_unique('custom_field_customfieldvalue', ['field_id', 'object_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'CustomFieldValue', fields ['field', 'object_id']
        db.delete_unique('custom_field_customfieldvalue', ['field_id', 'object_id'])

        # Removing unique constraint on 'CustomField', fields ['name', 'content_type']
        db.delete_unique('custom_field_customfield', ['name', 'content_type_id'])

        # Deleting model 'CustomField'
        db.delete_table('custom_field_customfield')

        # Deleting model 'CustomFieldValue'
        db.delete_table('custom_field_customfieldvalue')


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'custom_field.customfield': {
            'Meta': {'unique_together': "(('name', 'content_type'),)", 'object_name': 'CustomField'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'field_type': ('django.db.models.fields.CharField', [], {'default': "'t'", 'max_length': '1'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '75'})
        },
        'custom_field.customfieldvalue': {
            'Meta': {'unique_together': "(('field', 'object_id'),)", 'object_name': 'CustomFieldValue'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'field': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'instance'", 'to': "orm['custom_field.CustomField']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['custom_field']


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'CustomField.default_value'
        db.add_column(u'custom_field_customfield', 'default_value',
                      self.gf('django.db.models.fields.CharField')(default='', max_length=255, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'CustomField.default_value'
        db.delete_column(u'custom_field_customfield', 'default_value')


    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'custom_field.customfield': {
            'Meta': {'unique_together': "(('name', 'content_type'),)", 'object_name': 'CustomField'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'default_value': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'field_type': ('django.db.models.fields.CharField', [], {'default': "'t'", 'max_length': '1'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '75'})
        },
        u'custom_field.customfieldvalue': {
            'Meta': {'unique_together': "(('field', 'object_id'),)", 'object_name': 'CustomFieldValue'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'field': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'instance'", 'to': u"orm['custom_field.CustomField']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['custom_field']


from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.db import models
from django.contrib.contenttypes import generic

class CustomField(models.Model):
    """
    A field abstract -- it describe what the field is.  There are one of these
    for each custom field the user configures.
    """
    name = models.CharField(max_length=75)
    content_type = models.ForeignKey(ContentType)
    field_type = models.CharField(
        max_length=1, 
        choices=(('t','Text'),('i','Integer'),('b','Boolean (Yes/No)'),),
        default='t')
    default_value = models.CharField(
        max_length=255,
        blank=True,
        help_text="You may leave blank. For Boolean use True or False")

    def get_value_for_object(self, obj):
        return CustomFieldValue.objects.get_or_create(
            field=self,
            object_id=obj.id)[0]
    
    def __unicode__(self):
        return self.name
        
    class Meta:
        unique_together = ('name', 'content_type')
    

class CustomFieldValue(models.Model):
    """
    A field instance -- contains the actual data.  There are many of these, for
    each value that corresponds to a CustomField for a given model.
    """
    field = models.ForeignKey(CustomField, related_name='instance')
    value = models.CharField(max_length=255, blank=True, null=True)
    object_id = models.PositiveIntegerField()
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    
    def __unicode__(self):
        return self.value
    
    def save(self, *args, **kwargs):
        super(CustomFieldValue, self).save(*args, **kwargs)
        if not self.content_type:
            self.content_type = self.field.content_type
            self.save()

    def clean(self):
        """ Check value against field_type """
        if self.field.field_type == 'i' and self.value:
            try:
                # the float is to deal with things like '2.0'
                self.value = str(int(float(self.value)))
            except ValueError:
                raise ValidationError(
                    u'{0} must be an integer.'.format(self.field))
        elif self.field.field_type == 'b':
            if (str(self.value).lower() in ["t", "true", 'yes'] or
               self.value is True):
                self.value = "True"
            else:
                self.value = "False"
        return super(CustomFieldValue, self).clean()

    class Meta:
        unique_together = ('field', 'object_id')



from django import forms
from django.db import IntegrityError
from django.contrib.contenttypes.models import ContentType
from django.contrib.admin import ModelAdmin
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.contenttypes import generic
from django.core.exceptions import ValidationError

from .models import CustomField, CustomFieldValue

class Callable:
    def __init__(self, anycallable):
        self.__call__ = anycallable

class CustomFieldModel():
    """
    Abstract class adds some helper functions a Model
    """
    
    @property
    def get_custom_fields(self):
        """ Return a list of custom fields for this model """
        return CustomField.objects.filter(content_type=ContentType.objects.get_for_model(self))
    
    def get_model_custom_fields(self):
        """ Return a list of custom fields for this model, directly callable without an instance
        Use like Foo.get_model_custom_fields(Foo)
        """
        return CustomField.objects.filter(content_type=ContentType.objects.get_for_model(self))
    get_model_custom_fields = Callable(get_model_custom_fields)
        
    def get_custom_field(self, field_name):
        """ Get a custom field object for this model
        field_name - Name of the custom field you want.
        """
        content_type=ContentType.objects.get_for_model(self)
        return CustomField.objects.get(content_type=content_type,name=field_name)
        
    def get_custom_value(self, field_name):
        """ Get a value for a specified custom field
        field_name - Name of the custom field you want.
        """
        custom_field = self.get_custom_field(field_name)
        return CustomFieldValue.objects.get_or_create(field=custom_field,object_id=self.id)[0].value
        
    def set_custom_value(self, field_name, value):
        """ Set a value for a specified custom field
        field_name - Name of the custom field you want.
        value - Value to set it to
        """
        custom_field = self.get_custom_field(field_name)
        custom_value = CustomFieldValue.objects.get_or_create(field=custom_field,object_id=self.id)[0]
        custom_value.value = value
        custom_value.save()
    

class CustomFieldValueForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(CustomFieldValueForm, self).__init__(*args, **kwargs)
        if self.instance:
            try:
                if self.instance.field.field_type == "b":
                    check_choices = (('True', 'True'),('False', 'False'))
                    self.fields['value'] = forms.ChoiceField(choices=check_choices)
            except ObjectDoesNotExist:
                pass

from .models import CustomFieldValue
from django.contrib.contenttypes import generic
class CustomInline(generic.GenericTabularInline):
    model = CustomFieldValue
    form = CustomFieldValueForm
    can_delete = False
    readonly_fields = ('field',)
    fields = ('field', 'value')
    extra = 0
    max_num = 0

    def has_change_permission(self, request, obj=None):
        """ Need to always allow changing custom values """
        return True


class CustomFieldAdmin(ModelAdmin):
    """
    Abstract class addes functionality to deal with custom fields in Django admin.
    """
    def get_formsets(self, request, obj=None):
        for inline in self.get_inline_instances(request, obj):
            yield inline.get_formset(request, obj)

    def get_inline_instances(self, request, obj=None):
        inline_instances = []

        inlines = self.inlines
        if obj and CustomInline not in inlines:
            inlines += [CustomInline]

        for inline_class in inlines:
            inline = inline_class(self.model, self.admin_site)
            if request:
                if not isinstance(inline, CustomInline):
                    if not (inline.has_add_permission(request) or
                            inline.has_change_permission(request) or
                            inline.has_delete_permission(request)):
                        continue
                if not inline.has_add_permission(request):
                    inline.max_num = 0
            inline_instances.append(inline)
        return inline_instances 

    def get_form(self, request, obj=None, **kwargs):
        if obj:
            content_type=ContentType.objects.get_for_model(obj)
            custom_fields = CustomField.objects.filter(content_type=content_type)
            for custom_field in custom_fields:
                try:
                    field_value, created = CustomFieldValue.objects.get_or_create(
                        content_type = content_type,
                        object_id = obj.id,
                        field = custom_field,
                    )
                except IntegrityError:
                    # This can happen because content_type is really a
                    # cache field and didn't always exist
                    field_value, created = CustomFieldValue.objects.get_or_create(
                        object_id = obj.id,
                        field = custom_field,
                    )
                    field_value.content_type = content_type
                    field_value.save()
                if created:
                    if field_value.field.default_value:
                        field_value.value = field_value.field.default_value
                        field_value.save()
        return super(CustomFieldAdmin, self).get_form(request, obj, **kwargs)
